#!/usr/bin/env python

# Author                 Version      Date
# -----------------------------------------------
# Martin Still (SAAO)    1.0          06 Sep 2006

# saltkey...

from pyraf import iraf 
import saltprint, saltio, pyfits, string

# -----------------------------------------------------------
# get keyword value

def get(keyword,hdu,file,logfile):

    status = 0
    try:
	value = hdu.header[keyword]
    except:
	message = 'ERROR -- SALTKEY.GET: Cannot read keyword ' + keyword
	message += ' in file ' + file
	status = saltprint.err(logfile,message)
	value = None

    return value, status

# -----------------------------------------------------------
# does keyword exist

def exist(keyword,hdu,file,logfile):

    status = 0
    try:
	value = hdu.header[keyword]
    except:
	message = 'ERROR -- SALTKEY.EXIST: ' + keyword + ' does not exist'
	message += ' in file ' + file
	status = saltprint.err(logfile,message)

    return status

# -----------------------------------------------------------
# does keyword exist

def found(keyword,hdu):

    status = True
    try:
	value = hdu.header[keyword]
    except:
	status = False

    return status

# -----------------------------------------------------------
# does keyword match prediction

def match(keyword,value1,hdu,file,logfile):

    status = 0
    value2, status = get(keyword,hdu,file,logfile)
    if (status == 0 and value1 != value2):
	message  = 'ERROR -- SALTKEY.MATCH -- ' + file
	message += '[' + keyword + '] .not. = ' + value1 + ' : ERROR'
	status = saltprint.err(logfile,message)

    return status

# -----------------------------------------------------------
# change existing keyword value

def put(keyword,value,hdu,file,logfile):

    status = 0
    try:
	hdu.header.update(keyword,value)
    except:
	message = 'ERROR -- SALTKEY.PUT: Cannot update keyword ' + keyword
	message += ' in ' + file
	status = saltprint.err(logfile,message)

    return status

# -----------------------------------------------------------
# read keyword woth keypar IRAF tool i.e. without opening the whole file

def keypar(file,hdu,key,logfile):

    try:
	iraf.tables.keypar(file+'['+str(hdu)+']',key)
	value = iraf.tables.keypar.value
    except:
	message = 'ERROR -- SALTKEY.KEYPAR: Cannot read keyword ' + key
	message += ' in ' + file + '[' + str(hdu) + ']'
	status = saltprint.err(logfile,message)
	value = None

    return value

# -----------------------------------------------------------
# create keyword with mkheader IRAF tool i.e. without opening the whole file

def mkheader(file,keyword,value,comment,verbose,logfile):

    status = 0
    message = 'SALTKEY.MKHEADER: Created keyword ' + keyword + ' in ' + file
    try:
	tmpfile, status = saltio.tmpfile('.',False,logfile)
	tmp, status = saltio.openascii(tmpfile,'w',logfile)
	tmp.write('%-8s= \'%-18s\' / %-s\n' % (keyword,value,comment))
	status = saltio.closeascii(tmp,logfile)
	iraf.noao.artdata.mkheader(file,tmpfile,append='y',verbose='n')
	saltio.delete(tmpfile,False,logfile)
	saltprint.log(logfile,message,verbose)
    except:
	message = 'ERROR -- SALTKEY.MKHEADER: Cannot edit keyword ' + keyword
	message += ' in ' + file
	status = saltprint.err(logfile,message)

    return status

# -----------------------------------------------------------
# add new keyword

def new(keyword,value,comment,hdu,file,logfile):

    status = 0
    try:
	hdu.header.update(keyword,value,comment)
    except Exception, e:
	message = 'ERROR -- SALTKEY.NEW: Cannot create keyword %s in %s because %s ' % (keyword, file, e)
	status = saltprint.err(logfile,message)

    return status

# -----------------------------------------------------------
# delete keyword

def rem(keyword,hdu,file,logfile):

    status = 0
    try:
	del hdu.header[keyword]
    except:
	message = 'ERROR -- SALTKEY.DEL: Cannot delete keyword '+keyword
	message += ' in '+file
	status = saltprint.err(logfile,message)

    return status

# -----------------------------------------------------------
# observation date keyword

def dateobs(struct,file,logfile):

    status = 0
    year = ''
    month = ''
    day = ''
    try:
	date = struct.header['DATE-OBS']
	year = int(date.split('-')[0])
	month = int(date.split('-')[1])
	day = int(date.split('-')[2])
    except:
	message = 'ERROR -- SALTKEY.DATEOBS: DATE-OBS keyword not '
	message += 'recognized in file ' + file
	status = saltprint.err(logfile,message)
	
    return year, month, day, status

# -----------------------------------------------------------
# observation time keyword

def timeobs(struct,file,logfile):

    status = 0
    hour = ''
    minute = ''
    second = ''
    try:
	utcobs = struct.header['UTC-OBS']
	hour = int(utcobs.split(':')[0])
	minute = int(utcobs.split(':')[1])
	second = float(utcobs.split(':')[2])
    except:
	message = 'ERROR -- SALTKEY.TIMEOBS: UTC-OBS keyword not '
	message += 'recognized in file ' + file
	status = saltprint.err(logfile,message)
	
    return hour, minute, second, status

# -----------------------------------------------------------
# CCD binning keyword

def ccdbin(struct,file,logfile):

    status = 0
    xbin = 0
    ybin = 0
    try:
	ccdsum = struct.header['CCDSUM']
	xbin = int(ccdsum.split(' ')[0])
	ybin = int(ccdsum.split(' ')[1])
    except:
	message = 'ERROR -- SALTKEY.CCDBIN: CCDSUM keyword not recognized in file ' + file
	status = saltprint.err(logfile,message)
	
    return xbin, ybin, status

# -----------------------------------------------------------
# identify instrument in keywords

def instrumid(struct,file,logfile):

    instrume = ''
    keyprep = ''
    keygain = ''
    keybias = ''
    keyslot = ''
    status = 0
    try:
	instrume = struct[0].header['INSTRUME']
	if (string.join(instrume.split(),"") == 'RSS'):
	    keyprep = 'PPREPARE'
	    keygain = 'PGAIN'
	    keybias = 'PBIAS'
	    keyxtalk = 'PXTALK'
	    keyslot = 'PSLOT'
	elif (string.join(instrume.split(),"") == 'SALTICAM'):
	    keyprep = 'SPREPARE'
	    keygain = 'SGAIN'
	    keybias = 'SBIAS'
	    keyxtalk = 'SXTALK'
	    keyslot = 'SSLOT'
	else:
	    message = 'ERROR -- SALTKEY.INSTRUMID: INSTRUME keyword not '
	    message += 'recognized in file ' + file
	    status = saltprint.err(logfile,message)
    except:
	message = 'ERROR -- SALTKEY.INSTRUMID: INSTRUME keyword not found '
	message += 'in file ' + file
	status = saltprint.err(logfile,message)

    return instrume,keyprep,keygain,keybias,keyxtalk,keyslot,status

# -----------------------------------------------------------
# has file been prepared?

def prepare(struct,file,keyprep,logfile):

    status = 0
    try:
	prep = struct[0].header[keyprep]
    except:
	message = 'ERROR -- SALTIO.PREPARE: File ' + file + ' has not been prepared for '
	message += 'SALT IRAF/PyRAF tools'
	status = saltprint.err(logfile,message)

    return status

# -----------------------------------------------------------
# has file been cleaned?

def clean(struct,file,keyslot,keygain,logfile):

    status = 0
    try:
	slot = struct[0].header[keyslot]
    except:
	try:
	    gain = struct[0].header[keygain]
	except:
	    message = 'WARNING -- SALTIO.CLEAN File ' + file + ' is probably in '
	    message += 'a raw state'
	    status = saltprint.err(logfile,message)

    return status

# -----------------------------------------------------------
# add history keyword

def history(struct,message,file,logfile):

    import pyfits

    status = 0
    try:
	struct.header.add_history(message)
    except:
	message = 'ERROR -- SALTKEY.HISTORY: Cannot write HISTORY keyword to ' + file
	status = saltprint.err(logfile,message)

    return status

# -----------------------------------------------------------
# copy keyword

def copy(new,old,key,logfile):

    status = 0

    if found(key,old):
	try:
	    oldcard=old.header.ascardlist()
	    new.header.update(key,old.header[key],oldcard[key].comment)
	except:
            message  = 'ERROR -- SALTKEY.COPY: Cannot COPY KEYWORD ' + key 
            status = saltprint.err(logfile,message)


    return status

# -----------------------------------------------------------
# house cleaning keywords

def housekeeping(hdu, keytask, keycomment, hist, file,logfile):
    import time
    status = 0
    try:
        status = put('SAL-TLM',time.asctime(time.localtime()),hdu, file,logfile)
        status = new(keytask,time.asctime(time.localtime()),keycomment,hdu,file,logfile)
        status = history(hdu,hist,file,logfile)
    except Exception, e:
	message = 'ERROR--SALTKEY.HOUSECLEANING: Unable to append keywords because %s ' % e
	status = saltprint.err(logfile,message)
    return status
