#!/usr/bin/env python

# Author                 Version      Date
# -----------------------------------------------
# S M Crawford (SAA0)    0.30         16 Jul 2008
#
# SLOTBACK is a tool to produce to produce a fits file
# which only contains the fits extensions of interest.
# If selected, the tool will either create a background
# subtracted image or the background image.
#

import os,sys, time, re
from pyraf import iraf 
import saltprint, saltkey, saltio, salttime
import slottool
from slotbackground import subbackground

import numpy
import pyfits

# ----------------------------------------------------------
# core routine

def slotback(images,outfits,extension, imgtype, subbacktype, sigback, \
             mbin, sorder,  iter, ampperccd, ignorexp,      \
             clobber,logfile,verbose,status):



#set up the variables
    status = 0
    entries = []
    vig_lo = {}
    vig_hi = {}
    amp = {}
    x = {}
    y = {}
    x_o = {}
    y_o = {}
    r = {}
    br1 = {}
    br2 = {}
    hour = 0
    min = 0
    sec = 0.
    time0 = 0.
    nframes = 0
    bin=mbin
    order=sorder
    plotdata=False
    plotfreq=0.01
    ftime=plotfreq
    


# test the logfile

    logfile = saltio.logname(logfile)

# log the call 

    saltprint.line(logfile,verbose)
    message = 'SLOTBACK -- '
    message += 'images='+images+' '
    message += 'outfits='+outfits+' '
    message += 'extension=%i ' % extension
    message += 'imgtype='+imgtype+' '
    message += 'subbacktype='+subbacktype+' '
    message += 'sigback=%f ' % sigback
    message += 'bin=%i ' % bin
    message += 'iter=%i '% iter
    message += 'order=%i ' % order
    message += 'ampperccd=%i ' % ampperccd
    message += 'ignorexp=%i ' % ignorexp 
    yn = 'n'
    if (clobber): yn = 'y'
    message += 'clobber='+yn+' '
    message += 'logfile='+logfile+' '
    yn = 'n'
    if (verbose): yn = 'y'
    message += 'verbose='+yn+'\n'
    saltprint.log(logfile,message,verbose)

# start time

    saltprint.time('SLOTBACK started at  ',logfile,verbose)

# is the input file specified?
   
    if (status == 0): status = saltio.filedefined('Input',images,logfile)

# if the input file is a list, does it exist?

    if (status == 0 and images[0] == '@'): 
        status = saltio.listexists('Input',images,logfile)

# parse list of input files

    if (status == 0):
        infiles, status = saltio.listparse('Raw image',images,'','','',logfile)

# check input files exist

    if (status == 0): status = saltio.filesexist(infiles,'','r',logfile)

# is the output file specified?
   
    if (status == 0): status = saltio.filedefined('Output',outfits,logfile)

# check output file does not exist, optionally remove it if it does exist

    if (status == 0): 
	status = saltio.filedoesnotexist(outfits,'no',logfile)
        if (status == 1 and clobber): 
	  os.remove(outfits)
	  status = 0
        elif (status == 1 and not clobber):
	  message = 'ERROR -- SLOTBACK: File ' + outfits + ' already exists, use clobber=y'
	  status = saltprint.err(logfile,message)

# set the writenewfits parameter

    if status==0:
	if not outfits or outfits=='none':
	    writenewfits=False
	else:
	    writenewfits=outfits


# get time of first exposure and basic information about the observations


    if (status == 0):

	infile=infiles[0]
	struct, status = saltio.openfits(infile,logfile)

        # identify instrument
        if (status ==0 ):
	   instrume,keyprep,keygain,keybias,keyxtalk,keyslot,status = saltkey.instrumid(struct,infile,logfile)

	# how many extensions?
    	if (status == 0): 
		nextend,status = saltkey.get('NEXTEND',struct[0],infile,logfile)
		if nextend < extension:
		   message='SALTPHOT--ERROR:  Insufficient number of extensions in %s' % (infile)
		   status=saltprint.err(logfile, message)

	# how many amplifiers?
    	if (status == 0): 
	   amplifiers,status = saltkey.get('NCCDS',struct[0],infile,logfile)
	   amplifiers = int(ampperccd*float(amplifiers))
	   if ampperccd>0:
	      nframes = nextend/amplifiers
	      nstep=amplifiers
	   else:
	      nframes = nextend
	      nstep=1
	   ntotal=nframes*len(infiles)

	# image size
        if (status == 0):
	   naxis1,status = saltkey.get('NAXIS1',struct[extension],infile,logfile) 
	   naxis2,status = saltkey.get('NAXIS2',struct[extension],infile,logfile) 

	# CCD binning
        if (status == 0):
	  ccdsum,status = saltkey.get('CCDSUM',struct[0],infile,logfile) 
	  binx = int(ccdsum.split(' ')[0])
	  biny = int(ccdsum.split(' ')[1])



        # If a total file is to written out, create it and update it
	hdu = 1
        if status==0 and writenewfits:
	   #delete the file if clobber is on and the file exists
	   if (status == 0 and os.path.isfile(writenewfits)):
	     if clobber:
              status = saltio.delete(writenewfits,'no',logfile)
	     else:
	      message='ERROR -- SLOTBACK: Output fits file ' + writenewfits + 'already exists. '
	      message += 'Use Clobber=yes to overwrite file'
              status = saltprint.err(logfile,message)
	
	   #create the output file 
	   if status==0:
	    try:
              hdu = pyfits.PrimaryHDU()
              hdu.header=struct[0].header
	      hdu.header['NCCDS']=1
	      hdu.header['NSCIEXT']=ntotal-ignorexp
	      hdu.header['NEXTEND']=ntotal-ignorexp
              hduList = pyfits.HDUList(hdu)
              hduList.verify()
              hduList.writeto(writenewfits)
            except:
              message  = 'ERROR -- SLOTBACK: Could not create new fits file named ' + writenewfits
              status = saltprint.err(logfile,message)
	
	   #read it back in for updating
	   if status==0:
            try:
                  hduList = pyfits.open(writenewfits,mode='update')
            except:
                  message = 'ERROR -- SLOTBACK:  Canot open  ' + outfile
                  status = saltprint.err(logfile,message)

    #set up the completeness tracker
    if status==0 and verbose:
       j=0
       x2=float(j)/float(ntotal)
       ctext='Percentage Complete: %3.2f' % x2
       sys.stdout.write(ctext)
       sys.stdout.flush()


    if status==0:
      for infile in infiles:

	if (status == 0 ): struct, status = saltio.openfits(infile,logfile)

        # has file been cleaned?
        if (status == 0): cstatus=saltkey.clean(struct,infile,keyslot,keygain,logfile)

	#Skip through the frames and process each frame individually
	for i in range(nframes):
          if not (infile==infiles[0] and i < ignorexp):
	   pstatus=0
	   hdu=extension+i*nstep 
	   try:
	     header=struct[hdu].header
	     array=struct[hdu].data
	     array=array*1.0
	   except:
	     message='SLOTBACK--ERROR:  Unable to open extension %i in image %s' % (hdu, infile)
	     pstatus=saltprint.err(logfile,message)


	   #start the analysis of each frame
	   if pstatus==0:

             # gain and readout noise
	     try:
	       gain=float(header['GAIN'])
	     except:
	       gain=1
	       message='WARNING--SLOTBACK:  Gain not specified in image header'
	       saltprint.log(logfile,message,verbose)
	     try:
	       rdnoise=float(header['RDNOISE'])
	     except:
	       rdnoise=0
	       message='WARNING--SLOTBACK:  RDNOISE not specified in image header'
	       saltprint.log(logfile,message,verbose)

	   # background subtraction
	   if pstatus==0 and not subbacktype=='none':
	     narray,status = subbackground(array, sigback, bin, order, iter, subbacktype, logfile)

	   # create output array
	   if pstatus==0 and writenewfits:
	     try:
	       if imgtype=='background':
	         oarray=narray-array
	       else:
		 oarray=narray
	     except:
	       oarray=array

	   #plot data 
	   if pstatus==0 and verbose:
	     x2=float(j)/float(ntotal)
	     if x2 > ftime:
              ctext='\b\b\b\b\b %3.2f' % x2
              sys.stdout.write(ctext)
              sys.stdout.flush()
	      ftime += plotfreq
	
	   #write the fits file
	   if pstatus==0 and writenewfits: 
		hdue = pyfits.ImageHDU(oarray)
		hdue.header=header
		#update the header values with the image name and extension number
		try:
		     hdue.header.update('ONAME',infile,'Original image name')
		     hdue.header.update('OEXT',hdu,'Original extension number')

		except:
		     message='SALTPHOT--WARNING:  Could not update image in newfits file for %s ext %i' % (infile, hdu)
	             saltprint.log(logfile,message,verbose)
		hduList.append(hdue)

	   j +=1
	   if status==1: print j

        # close FITS file		
	status = saltio.closefits(struct,logfile)

    print('')

# close the output fits file:
	
    if status==0 and writenewfits:
	  try:
	   #write out the file
           hduList.flush()
           hduList.close()
          except:
           message = 'ERROR: SLOTBACK-- Failed to write ' + writenewfits
           status = saltprint.err(logfile,message)


# end time

    if (status == 0):
	saltprint.time('SLOTBACK completed at',logfile,verbose)
    else:
	saltprint.time('SLOTBACK aborted at',logfile,verbose)


# -----------------------------------------------------------
# main code 

parfile = iraf.osfn("slottools$slotback.par") 
t = iraf.IrafTaskFactory(taskname="slotback",value=parfile,function=slotback,pkgname='slottools')
